disp(' ');
disp('  mdof_enforced_acceleration_nm.m  ver 1.7  November 21, 2013 ');
disp('  by Tom Irvine ');
%
disp(' ');
disp('  Response of a multi-degree-of-freedom system to enforced ');
disp('  acceleration at selected point mass nodes via the Newmark-beta ');
disp('  method. ');
disp(' ');
%
close all hidden;
%
clear Mdd;  % driven
clear Mdf;
clear Mfd;
clear Mff;  % free
clear Mdw;
clear Mwd;
clear Mww;
%
clear Cdw;
clear Cwd;
clear Cww;
clear Cdd;  
clear Cdf;
clear Cfd;
clear Cff;
%
clear Kdw;
clear Kwd;
clear Kww;
clear Kdd;  
clear Kdf;
clear Kfd;
clear Kff;
%
clear invKff;
%
clear MP;
clear MT;
clear CP;
clear CT;
clear KP;
clear KT;
%
clear mass;
clear cdamping;
clear stiff;
%
clear n;
clear na;
clear nv;
clear nd;
%
clear m;
clear c;
clear k;
%
clear TT;
clear T1;
clear T2;
%
clear I;
clear Idd;
clear Iff;
%
clear damp;
clear ea;
clear f;
clear ngw;
%
clear ttend;
clear ttstart;
clear length;
%
clear inv_KH;
clear inv_KH_diag;
%
clear dT;
clear vT;
clear accT;
%
clear ZdT;
clear ZvT;
clear ZaccT;
%
clear dvelox;
clear ddisp;
%
clear forig;
%
disp(' ');
disp(' Enter the units system ');
disp('  1=in/sec^2, in/sec, in ');
disp('  2=G, in/sec, in ');
disp('  3=m/sec^2, m/sec, mm/sec ');
%
iu=input(' ');
%
disp(' Assume symmetric mass and stiffness matrices. ');
%
mass_scale=1;
%
if(iu<=2)
     disp(' Select input mass unit ');
     disp('  1=lbm  2=lbf sec^2/in  ');
     imu=input(' ');
     if(imu==1)
         mass_scale=386;
     end
else
    disp(' mass unit = kg ');
end
%
if(iu<=2)
    disp(' stiffness unit = lbf/in ');
else
    disp(' stiffness unit = N/m ');
end
%
disp(' ');
disp(' Select file input method ');
disp('   1=file preloaded into Matlab ');
disp('   2=Excel file ');
file_choice = input('');
%
disp(' ');
disp(' Mass Matrix ');
%
if(file_choice==1)
        m = input(' Enter the matrix name:  ');
end
if(file_choice==2)
        [filename, pathname] = uigetfile('*.*');
        xfile = fullfile(pathname, filename);
%
        m = xlsread(xfile);
%
end
%
m=m/mass_scale;
%
mass=m;
num=max(size(m));
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
disp(' ');
disp(' Damping Coefficient Matrix ');
%
if(file_choice==1)
        c = input(' Enter the matrix name:  ');
end
if(file_choice==2)
        [filename, pathname] = uigetfile('*.*');
        xfile = fullfile(pathname, filename);
%
        c = xlsread(xfile);
%
end
cdamping=c;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
disp(' ');
disp(' Stiffness Matrix ');
%
if(file_choice==1)
        k = input(' Enter the matrix name:  ');
end
if(file_choice==2)
        [filename, pathname] = uigetfile('*.*');
        xfile = fullfile(pathname, filename);
%
        k = xlsread(xfile);
%
end
stiff=k;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
sz=size(mass);
ndof=(sz(1));
%
if(ndof<=20)
    disp(' Input Matrices ');
    mass
    cdamping
    stiff
end
%
if(ndof<=30)
    [fn,omega,ModeShapes,MST]=Generalized_Eigen(stiff,mass,1);
else
    [fn,omega,ModeShapes,MST]=Generalized_Eigen(stiff,mass,2);    
end    
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

num=ndof;
%
out1=sprintf('\n number of dofs =%d \n',num);
disp(out1);
%
disp(' ');
disp(' Enter the duration (sec) ');
dur=input(' ');
disp(' ');
sug=20*(max(omega)/(2*pi));
%
disp(' Enter the sample rate (samples/sec)');
out1=sprintf(' (suggest > %8.4g ) ',sug);
disp(out1);
sr=input(' ');
dt=1/sr;
nt=1+round(dur/dt);
%
t =  linspace(0,dur,nt);
nt=length(t);
t=t';
%
[FFI,accel_dof,nff,nem,nfree,enforced_dof,free_dof,ngw]=...
    ODE_acceleration_input(iu,num,nt,t,file_choice);
%
forig=zeros(nt,nem);
%
k=1;
for i=1:num
    nfi=accel_dof(i);
    if(nfi>0)
        forig(:,k)=FFI(:,nfi);
        k=k+1;
    end    
end
%
clear FFI;
FFI=forig;
%
for i=1:nem
    A=FFI(:,i);
    dv=detrend(cumtrapz(A));
    dvelox(:,i)=dv*dt;
end
%
for i=1:nem
    A=dvelox(:,i);
    dd=detrend(cumtrapz(A));
    ddisp(:,i)=dd*dt;    
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Partition Mass & Stiffness Matrices
%
disp(' ');
disp(' Display partitioned matrices?  1=yes  2=no ');
dispm=input(' ');
%
[fn,omega,ModeShapes,MST,Mwd,Cwd,Kwd,Mww,Cww,Kww,TT,T1,T2,KT,CT,MT]=...
   partition_matrices_mck(mass,cdamping,stiff,num,nfree,nem,free_dof,...
                                               enforced_dof,'accel',dispm);
%
omegan=omega;
%
clear r;
clear part;
clear ModalMass;
%
sz=size(Mww);
%
n_free_dof=sz(1);
%
num_modes=n_free_dof;
%
clear temp;
temp=ModeShapes(:,1:num_modes);
clear ModeShapes;
ModeShapes=temp;
MST=ModeShapes';
%
r=ones(num_modes,1);
%
part = MST*Mww*r;
%
if(size(part)<20)
    disp(' Participation Factors  ');
    part
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  displacement
%
%  d
%  v
%  acc
%
%%
n=num_modes;
%
M=Mww;
C=Cww;
K=Kww;
%
%%
[a0,a1,a2,a3,a4,a5,a6,a7]=Newmark_coefficients(dt);
%
KH=K+a0*M+a1*C;
%
inv_KH=pinv(KH); 
%
clear d;
clear v;
clear acc;
%
t=zeros(nt,1);
d=zeros(nt,nfree);
v=zeros(nt,nfree);
acc=zeros(nt,nfree);
%
U=zeros(n,1);
Ud=zeros(n,1);
Udd=zeros(n,1);
%
jj=1;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ns=size(U);
if(ns(2)>ns(1))
    U=U';
end
ns=size(Ud);
if(ns(2)>ns(1))
    Ud=Ud';
end
ns=size(Udd);
if(ns(2)>ns(1))
    Udd=Udd';
end
%%
for i=1:(nt-1)
%   
    FP=-Mwd*(FFI(i,:))';
%    
    FH=FP+M*(a0*U+a2*Ud+a3*Udd)+C*(a1*U+a4*Ud+a5*Udd);
%
%%    Un=KH\FH;
%
    Un=inv_KH*FH;
%
    Uddn=a0*(Un-U)-a2*Ud-a3*Udd;
    Udn=Ud+a6*Udd+a7*Uddn;
    U=Un;
    Ud=Udn;
    Udd=Uddn;
%
    d(i+1,:)=U';
    v(i+1,:)=Ud';
    acc(i+1,:)=Udd';
    t(i+1)=i*dt;   
%
end    
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
clear d_save;
clear v_save;
clear a_save;
d_save=d;
v_save=v;
a_save=acc;
%
%    Transformation back to xd xf
%
%%    dT=TT*d';
%%    vT=TT*v';
%%    accT=TT*acc';
%
dT=(T1*ddisp' + T2*d')';
vT=(T1*dvelox' + T2*v')';
accT=(T1*FFI' + T2*acc')';
%
%  Put in original order
%
ZdT=[ ddisp  dT ];
ZvT=[ dvelox  vT ];
ZaccT=[ forig  accT ];
%
clear d;   
clear v;
clear acc;
d=zeros(nt,num);
v=zeros(nt,num);
acc=zeros(nt,num);
%
for i=1:num
   for j=1:nt
        d(j,ngw(i))=  (ZdT(j,i));        
        v(j,ngw(i))=  (ZvT(j,i));     
      acc(j,ngw(i))=(ZaccT(j,i));
   end
end
%
disp(' ');
disp(' Detrend velocity & displacement?  1=yes 2=no ');
idet=input(' ');
%
if(idet==1)
    for i=1:num
        p = polyfit(t,d(:,i),2);
        d(:,i)=d(:,i)-p(1)*t.^2 -p(2)*t -p(3);
        p = polyfit(t,d(:,i),0);  % use instead of detrend
        v(:,i)=v(:,i)-p(1);           
    end
end
%
if(iu==2)
    acc=acc/386;
end
%
if(iu==3)
    d=d/1000;
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
mdof_plot_ea(t,d,v,acc,num,iu)
%